# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2018 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        mssqldb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     13/08/2018
# ------------------------------------------------------------------------------
import os,sys,re
import pyodbc
from abstractdb import *
from gecosistema_core import *


def __readmssqlstring__(text):
    """
    __readmssqlstring__

        "dbname='Catasto' host=.\SQLEXPRESS user='sa' password='12345' srid=4326 type=MultiPolygon table="dbo"."SearchComboBox" (Geometry) sql="  from .qgs file
        "MSSQL:server=.\SQLEXPRESS;trusted_connection=no;uid=sa;pwd=12345;database=Catasto;"   MSSQLSpatial dns
        "DRIVER={SQL Server};SERVER=.\SQLEXPRESS;PORT=1433;DATABASE=Catasto;uid=sa;pwd=12345"  ODBC

    """
    env = {}
    DICTIONARY = {
        "host": "server",
        "user": "uid",
        "password": "pwd",
        "dbname": "database",
        "table": "tablename",
        "type": "geometry_type"
    }
    text = text.replace("MSSQL:", "")

    arr = listify(text, " ;")
    for item in arr:
        item = item.split("=", 1)
        if len(item) > 1:
            key = DICTIONARY[item[0]] if item[0] in DICTIONARY else item[0]
            env[key] = ("%s" % item[1]).strip("'")
    # some correction
    if "tablename" in env and "." in env["tablename"]:
        text = chrtran(env["tablename"], '"', '')
        env["catalog"], env["tablename"] = text.split(".", 1)
    return env


class mssqlDB(AbstractDB):
    """
    mssqlDB - Microsoft Sql database wrapper
    """
    version = "2.0.0"

    def __init__(self, filename="", modules="", verbose=False):
        """
        Constructor
        """
        if isinstance(filename, dict):
            self.__env__ = filename
        elif isstring(filename) and isfile(filename):
            self.__env__ = readmssql(dsn)
        elif isstring(filename):
            self.__env__ = __readmssqlstring__(filename)
        else:
            raise Exception("Unable to initialize the mssql connection!")

        self.dsn = sformat("""DRIVER={SQL Server};SERVER={server};PORT=1433;DATABASE={database};uid={uid};pwd={pwd}""",self.__env__)
        AbstractDB.__init__(self, self.dsn )

    def __connect__(self):
        """
        Connect to the mysql instance
        """
        try:
            self.conn = pyodbc.connect(self.dsn)

        except pyodbc.Error as err:
            print(err)
            self.close()


    def GetTables(self, like="%", verbose=False):
        """
        GetTables - Return a list with all tablenames
        """
        env = {"like":like}
        tables = self.execute("""SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE='BASE TABLE' AND TABLE_NAME LIKE '{like}';""",env,outputmode="array",verbose=verbose)
        return [item for (item,) in tables]

    def GetFieldNames(self, tablename, ctype="%", typeinfo=False):
        """
        GetFieldNames
        """
        env = {"tablename":tablename, "ctype":ctype}

        #select   COLUMN_NAME, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION, DATETIME_PRECISION,IS_NULLABLE
        #from INFORMATION_SCHEMA.COLUMNS
        #where TABLE_NAME = 'civici'
        if typeinfo:
            sql = """SELECT COLUMN_NAME, DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME LIKE '{tablename}' AND DATA_TYPE LIKE '{ctype}';"""
        else:
            sql = """SELECT COLUMN_NAME            FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME LIKE '{tablename}' AND DATA_TYPE LIKE '{ctype}';"""
        return self.execute(sql, env, outputmode="array", verbose=True)


    def insertMany(self, tablename, values, commit=True, verbose=False):
        """
        insertMany
        """
        #TODO

    def Execute(text, env=None, outputmode="cursor", verbose=False):
        """
        Execute
        """
        # TODO

    Execute = staticmethod(Execute)

if __name__ == "__main__":
    env = {
        "server": r".\SQLEXPRESS",
        "uid": r"sa",
        "pwd": r"12345",
        "database": "Catasto",
        "tablename": "civici"
    }

    db = mssqlDB(env)

    cursor = db.execute("""SELECT TOP(5) comune FROM  [{database}].[dbo].[{tablename}]""", env, verbose=True)
    #print(cursor)

    filename = Desktop()+"/civici.csv"
    #print db.toCsv(filename, tables="civici", sep=";", decimal=".", verbose=True)

    print db.toExcel(filename, tables="civici", sep=";", decimal=".", verbose=True)


import os
import setuptools

version = "0.0.0"
if os.path.isfile("version.txt"):
    with open("version.txt", "r") as f:
        version = f.read()
        if version:
            versionno = version.split("=")[-1].strip("' ")
            arr = [int(item) for item in versionno.split(".")]
            arr[2] += 1
            version = "%d.%d.%d" % tuple(arr)
            with open("version.txt", "w") as w:
                w.write(version)

setuptools.setup(
    name="gecosistema_database",
    version=version,
    author="Valerio Luzzi",
    author_email="valluzzi@gmail.com",
    description="A small example package",
    long_description="A small example package",
    url="https://github.com/valluzzi/gecosistema_database.git",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    #apt-get install unixodbc
    install_requires=['gecosistema_core','unicodecsv','pyodbc','numpy','scipy']
)



# -------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2018 Luzzi Valerio 
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        aggregates.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     28/08/2018
# -------------------------------------------------------------------------------
import numpy as np
from scipy import stats

class Ensemble:
    """
    Ensemble
    """
    def __init__(self):
        self.p = 0.5
        self.values  = []

    def step(self, value,probability):
        if (value!=None):
            self.p = probability
            self.values.append(value)

    def finalize(self):
        self.values = np.sort(self.values)
        mean,std = np.mean(self.values),np.std(self.values)
        cdf = stats.norm.cdf(self.values,mean,std)
        for j in range(len(cdf)):
            if cdf[j]>self.p:
                return self.values[j]
        return self.values[-1] if len(self.values)>0 else 0.0


# -------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2018 Luzzi Valerio 
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        sql_utils.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     05/09/2018
# -------------------------------------------------------------------------------
from gecosistema_core import *
import xlrd

SQLTYPES = {
    9999: "",
    9998: "EMPTY",
    1: "TEXT",
    2: "DATETIME",
    3: "DATE",
    4: "TIME",
    5: "FLOAT",
    6: "INTEGER",
    7: "GEOMETRY",
    "": 9999,
    "EMPTY": 9998,
    "TEXT": 1,
    "DATETIME": 2,
    "DATE": 3,
    "TIME": 4,
    "FLOAT": 5,
    "INTEGER": 6,
    "GEOMETRY": 7
}

def sqltype(cvalue, ctype=xlrd.XL_CELL_TEXT, nodata=("", "Na", "NaN", "-", "--", "N/A")):
    """
    Type symbol	Type number	Python value
    XL_CELL_EMPTY	0	empty string u''
    XL_CELL_TEXT	1	a Unicode string
    XL_CELL_NUMBER	2	float
    XL_CELL_DATE	3	float
    XL_CELL_BOOLEAN	4	int; 1 means TRUE, 0 means FALSE
    XL_CELL_ERROR	5	int representing internal Excel codes; for a text representation, refer to the supplied dictionary error_text_from_code
    XL_CELL_BLANK	6	empty string u''. Note: this type will appear only when open_workbook(..., formatting_info=True) is used.
    """
    if isarray(cvalue):
        if not isarray(ctype):
            ctype = [ctype] * len(cvalue)
        return [sqltype(cv, ct, nodata) for cv, ct in zip(cvalue, ctype)]
    if isinstance(cvalue, xlrd.sheet.Cell):
        return sqltype(cvalue.value, cvalue.ctype, nodata)
    if ctype == xlrd.XL_CELL_EMPTY:
        return 'EMPTY'
    elif ctype == xlrd.XL_CELL_TEXT and cvalue in nodata:
        return ''
    elif ctype == xlrd.XL_CELL_TEXT and isdate(cvalue):
        return 'DATE'
    elif ctype == xlrd.XL_CELL_TEXT and isdatetime(cvalue):
        return 'DATETIME'
    elif ctype == xlrd.XL_CELL_TEXT and parseInt(cvalue):
        return 'INTEGER'
    elif ctype == xlrd.XL_CELL_TEXT and parseFloat(cvalue) != None:
        return 'FLOAT'
    elif ctype == xlrd.XL_CELL_TEXT:
        return 'TEXT'
    elif ctype == xlrd.XL_CELL_NUMBER and int(cvalue) == cvalue:
        return 'INTEGER'
    elif ctype == xlrd.XL_CELL_NUMBER:
        return 'FLOAT'
    elif ctype == xlrd.XL_CELL_DATE:
        return 'DATETIME'
    elif ctype == xlrd.XL_CELL_BOOLEAN:
        return 'INTEGER'
    elif ctype == xlrd.XL_CELL_ERROR:
        return ''
    elif ctype == xlrd.XL_CELL_BLANK:
        return ''
    else:
        return 'TEXT'

def splitby(pattern, text, flags=0):
    """
    splitby -  split text by pattern
    """
    res = []
    idxs = [0]
    p = re.compile(pattern, flags)
    for m in p.finditer(text):
        #print m.start(), m.group()
        idxs+= [m.start()]
    idxs+=[len(text)+1]
    for j in range(1,len(idxs)):
        start = idxs[j-1]
        end = idxs[j]
        res.append( text[start:end])
    return res